{% load static %}
<link href="{% static 'viewflow/fonts/roboto/roboto-fontface.css' %}" rel="stylesheet">
<link href="{% static 'viewflow/fonts/material-icons/material-icons.css' %}" rel="stylesheet">
<link href="{% static 'viewflow/css/viewflow.min.css' %}" rel="stylesheet">
<link href="{% static 'viewflow/css/perfect-scrollbar.css' %}" rel="stylesheet">
