{% extends 'viewflow/base_lockscreen.html' %}
{% load i18n viewflow %}

{% block title %}{{ title }} - {{ block.super }}{% endblock %}
{% block lockscreen-sidebar-header-title %}{{ title }}{% endblock %}


{% block lockscreen-sidebar-content %}

  {% if validlink %}

    <p>{% trans "Please enter your new password twice so we can verify you typed it in correctly." %}</p>

    <vf-form>
      <form class="vf-form" method="POST" {% if form.is_multipart %} enctype="multipart/form-data"{% endif %}>
        {% csrf_token %}
        {% form form view.layout %}
        <div class="vf-page-lockscreen__sidebar-action-row">
          <button class="mdc-button mdc-card__action mdc-card__action--button mdc-button--raised" type="submit">{% trans 'Change my password' %}</button>
        </div>
      </form>
    </vf-form>

  {% else %}

    <p>{% trans "The password reset link was invalid, possibly because it has already been used.  Please request a new password reset." %}</p>

  {% endif %}

{% endblock %}
