{% extends 'viewflow/base_lockscreen.html' %}
{% load i18n %}

{% block title %}{{ title }} - {{ block.super }}{% endblock %}
{% block lockscreen-sidebar-header-title %}{{ title }}{% endblock %}


{% block lockscreen-sidebar-content %}
  <p>{% trans 'We’ve emailed you instructions for setting your password, if an account exists with the email you entered. You should receive them shortly.' %}</p>
  <p>{% trans 'If you don’t receive an email, please make sure you’ve entered the address you registered with, and check your spam folder.' %}</p>
  <a class="mdc-button mdc-button--raised vf-page-lockscreen__sidebar-content-action" href="/">
    {% trans 'View site' %}
  </a>
{% endblock %}

{% block lockscreen-content-icon %}<i class="logo__icon material-icons">lock</i>{% endblock %}

