{% extends 'viewflow/base_lockscreen.html' %}
{% load i18n %}

{% block title %}{{ title }} - {{ block.super }}{% endblock %}
{% block lockscreen-sidebar-header-title %}{{ title }}{% endblock %}


{% block lockscreen-sidebar-content %}

    <p>{% trans "Your password has been set.  You may go ahead and log in now." %}</p>

    <p><a class="mdc-button mdc-button--raised vf-page-lockscreen__sidebar-content-action" href="{{ login_url }}">{% trans 'Log in' %}</a></p>

{% endblock %}

{% block lockscreen-content-icon %}<i class="logo__icon material-icons">lock_open</i>{% endblock %}

